import 'package:awesome_notifications/awesome_notifications.dart';
import 'package:feedify/feedsdk.dart';
import 'package:feedify/src/manager/feed_api_manager.dart';
import 'package:feedify/src/model/notification/model_notification.dart';
import 'package:feedify/src/utils/notification_util.dart';
import 'package:flutter/material.dart';
import 'package:url_launcher/url_launcher.dart';

import '../../const.dart';
import '../../logs.dart';
import '../storage/app-prefs.dart';

class NotificationManager {
  static final NotificationManager _singleton = NotificationManager._internal();
  Button1? _button1;
  Button2? _button2;

  static AppPreferences prefs = AppPreferences();

  NotificationManager._internal();

  static NotificationManager getInstance() {
    return _singleton;
  }

  void init(Function(Map<String, String?>)? notificationDataCallback) async {
    await _initialiseNotifications();
    _initialiseNotificationsClick(notificationDataCallback);
  }

  Future<bool> _initialiseNotifications() async {
    return await AwesomeNotifications().initialize(
      FeedSDK.notificationIco,
      [
        NotificationChannel(
            importance: NotificationImportance.High,
            channelGroupKey: 'FEEDIFY_CHANNEL',
            channelKey: 'basic_channel',
            channelName: 'FEEDIFY_CHANNEL',
            channelDescription: 'Feed notification channel.',
            defaultColor: Colors.white,
            channelShowBadge: true,
            ledColor: Colors.white)
      ],
      debug: true,
    );
  }

  void _initialiseNotificationsClick(
      Function(Map<String, String?>)? notificationDataCallback) {
    AwesomeNotifications().setListeners(
        onActionReceivedMethod: (ReceivedAction receivedAction) async {
      Logs.d(
          'Notification clicked id ::  ${receivedAction.id}, button pressed::  ${receivedAction.buttonKeyPressed}');
      var map = receivedAction.payload;
      if (map != null) {
        if (map[Constants.type] == 'ad') {
          _launchURL(map[Constants.url]);
        } else {
          if (notificationDataCallback == null) {
            // handle it here
            if (receivedAction.buttonKeyPressed.isEmpty) {
              // clicked on notification part not on buttons
              _launchURL(map[Constants.url]);
            } else {
              if (receivedAction.buttonKeyPressed == _button1?.title) {
                _launchURL(_button1?.action);
              } else if (receivedAction.buttonKeyPressed == _button2?.title) {
                _launchURL(_button2?.action);
              }
            }
          } else {
                Map<String, String?> data = await prefs.getNotificationsData();
            NotificationUtils.notificationPayloadKeys.clear();
            notificationDataCallback.call(data);
          }
        }
        _callClickApi(map);
      }
    });
  }

  void showNotification(final ModelNotification notificationData) {
    Logs.d('Notification clicked notificationData ::  $notificationData');
    if (notificationData.id != null &&
        notificationData.id?.isNotEmpty == true) {
      bool isBigPicture = false;
      if (notificationData.image != null &&
          notificationData.image!.isNotEmpty) {
        isBigPicture = true;
      }
      AwesomeNotifications().createNotification(
        content: NotificationContent(
          notificationLayout: isBigPicture
              ? NotificationLayout.BigPicture
              : NotificationLayout.BigText,
          id: int.parse(notificationData.id!),
          channelKey: 'basic_channel',
          title: notificationData.title,
          body: notificationData.body,
          autoDismissible: true,
          largeIcon: notificationData.icon,
          bigPicture: notificationData.image,
          payload: NotificationUtils.getInstance()
              .getNotificationData(notificationData),
        ),
        actionButtons: _getActionButtons(notificationData),
      );
    }
  }

  List<NotificationActionButton> _getActionButtons(
      ModelNotification notificationData) {
    List<NotificationActionButton> actionButtons = [];
    _button1 = notificationData.button?.button1;
    _button2 = notificationData.button?.button2;

    if (_button1 != null && _button2 != null) {
      if (_button1?.title?.isNotEmpty == true) {
        NotificationActionButton actionButton = NotificationActionButton(
            key: _button1!.title!,
            label: _button1!.title!,
            icon: _button1!.icon,
            autoDismissible: true,
            actionType: ActionType.Default);
        actionButtons.add(actionButton);
      }
      if (_button2?.title?.isNotEmpty == true) {
        NotificationActionButton actionButton = NotificationActionButton(
            key: _button2!.title!,
            label: _button2!.title!,
            icon: _button2!.icon,
            autoDismissible: true,
            actionType: ActionType.Default);
        actionButtons.add(actionButton);
      }
    }
    return actionButtons;
  }

  void _launchURL(String? url) async {
    if (url != null) {
      if (!url.startsWith('http')) {
        url = 'http://' + url;
      }
      url = Uri.encodeFull(url);
      if (await canLaunch(url)) {
        await launch(url);
      } else {
        Logs.d('Could not launch $url');
      }
    } else {
      Logs.d('Could not launch $url');
    }
  }

  void _callClickApi(Map<String, String?> map) {
    var type = map[Constants.type];
    var notificationId = map[Constants.notificationId];

    if (type != null &&
        type.isNotEmpty == true &&
        notificationId != null &&
        notificationId.isNotEmpty == true) {
      FeedApiManager.getInstance()
          .callClickApi(map[Constants.type]!, map[Constants.notificationId]!);
    }
  }
}
