import 'dart:convert';

import 'package:feedify/logs.dart';

class ModelNotification {
  ModelNotification({
    String? title,
    String? body,
    String? url,
    String? icon,
    String? image,
    Button? button,
    String? button2,
    String? id,
    String? type,
    List<CustomParams>? customParams,
    List<CustomParamsBtn>? customParamsBtn,
  }) {
    _title = title;
    _body = body;
    _url = url;
    _icon = icon;
    _image = image;
    _button = button;
    _button2 = button2;
    _id = id;
    _type = type;
    _customParams = customParams;
    _customParamsBtn = customParamsBtn;
  }

  ModelNotification.fromJson(dynamic json) {
    _title = json['title'];
    _body = json['body'];
    _url = json['url'];
    _icon = json['icon'];
    _image = json['image'];
    try {
      _button = json['button'] != null && json["button"] != ""
          ? Button.fromJson(jsonDecode(json['button']))
          : null;
    } catch (e) {
      _button = json['button'] != null && json["button"] != ""
          ? Button.fromJson((json['button']))
          : null;
    }
    try {
      _button2 = json['button2'];
    } catch (e) {
      Logs.d(e.toString());
    }
    _id = json['id'];
    _type = json['type'];
    try {
      if (jsonDecode(json['custom_params']) != null) {
        _customParams = [];
        jsonDecode(json['custom_params']).forEach((v) {
          _customParams?.add(CustomParams.fromJson(v));
        });
      }
    } catch (e) {
      Logs.d(e.toString());
    }

    try {
      if (jsonDecode(json['custom_params_btn']) != null) {
        _customParamsBtn = [];
        jsonDecode(json['custom_params_btn']).forEach((v) {
          customParamsBtn?.add(CustomParamsBtn.fromJson(v));
        });
      }
    } catch (e) {
      Logs.d(e.toString());
    }
  }

  String? _title;
  String? _body;
  String? _url;
  String? _icon;
  String? _image;
  Button? _button;
  String? _button2;
  String? _id;
  String? _type;
  List<CustomParams>? _customParams;
  List<CustomParamsBtn>? _customParamsBtn;

  String? get title => _title;

  String? get body => _body;

  String? get url => _url;

  String? get icon => _icon;

  String? get image => _image;

  Button? get button => _button;

  String? get button2 => _button2;

  String? get id => _id;

  String? get type => _type;

  List<CustomParams>? get customParams => _customParams;

  List<CustomParamsBtn>? get customParamsBtn => _customParamsBtn;

  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    map['title'] = _title;
    map['body'] = _body;
    map['url'] = _url;
    map['icon'] = _icon;
    map['image'] = _image;
    try {
      if (_button != null) {
        map['button'] = _button?.toJson();
      }
    } catch (e) {
      Logs.d(e.toString());
    }
    try {
      map['button2'] = _button2;
    } catch (e) {
      Logs.d(e.toString());
    }
    map['id'] = _id;
    map['type'] = _type;
    try {
      map['custom_params'] = _customParams?.map((v) => v.toJson()).toList();
    } catch (e) {
      Logs.d(e.toString());
    }
    try {
      map['custom_params_btn'] =
          _customParamsBtn?.map((v) => v.toJson()).toList();
    } catch (e) {
      Logs.d(e.toString());
    }
    return map;
  }

  @override
  String toString() {
    return 'title :: $title, body :: $body, id :: $id, type :: $type ';
  }
}

class CustomParamsBtn {
  CustomParamsBtn({
    String? key,
    String? value,
    String? icon,
  }) {
    _key = key;
    _value = value;
    _icon = icon;
  }

  CustomParamsBtn.fromJson(dynamic json) {
    _key = json['key'];
    _value = json['value'];
    _icon = json['icon'];
  }

  String? _key;
  String? _value;
  String? _icon;

  String? get key => _key;

  String? get value => _value;

  String? get icon => _icon;

  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    map['key'] = _key;
    map['value'] = _value;
    map['icon'] = _icon;
    return map;
  }
}

class CustomParams {
  CustomParams({
    String? key,
    String? value,
  }) {
    _key = key;
    _value = value;
  }

  CustomParams.fromJson(dynamic json) {
    _key = json['key'];
    _value = json['value'];
  }

  String? _key;
  String? _value;

  String? get key => _key;

  String? get value => _value;

  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    map['key'] = _key;
    map['value'] = _value;
    return map;
  }
}

class Button {
  Button({
    Button1? button1,
    Button2? button2,
  }) {
    _button1 = button1;
    _button2 = button2;
  }

  Button.fromJson(dynamic json) {
    _button1 = json['button1'] != null && json["button1"] != ""
        ? Button1.fromJson(json['button1'])
        : null;
    _button2 = json['button2'] != null && json["button2"] != ""
        ? Button2.fromJson(json['button2'])
        : null;
  }

  Button1? _button1;
  Button2? _button2;

  Button1? get button1 => _button1;

  Button2? get button2 => _button2;

  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    if (_button1 != null) {
      map['button1'] = _button1?.toJson();
    }
    if (_button2 != null) {
      map['button2'] = _button2?.toJson();
    }
    return map;
  }
}

class Button2 {
  Button2({
    String? title,
    String? action,
    String? icon,
  }) {
    _title = title;
    _action = action;
    _icon = icon;
  }

  Button2.fromJson(dynamic json) {
    _title = json['title'];
    _action = json['action'];
    _icon = json['icon'];
  }

  String? _title;
  String? _action;
  String? _icon;

  String? get title => _title;

  String? get action => _action;

  String? get icon => _icon;

  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    map['title'] = _title;
    map['action'] = _action;
    map['icon'] = _icon;
    return map;
  }
}

class Button1 {
  Button1({
    String? title,
    String? action,
    String? icon,
  }) {
    _title = title;
    _action = action;
    _icon = icon;
  }

  Button1.fromJson(dynamic json) {
    _title = json['title'];
    _action = json['action'];
    _icon = json['icon'];
  }

  String? _title;
  String? _action;
  String? _icon;

  String? get title => _title;

  String? get action => _action;

  String? get icon => _icon;

  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    map['title'] = _title;
    map['action'] = _action;
    map['icon'] = _icon;
    return map;
  }
}
